package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.result.TBQualityProblemApplyResult;
import com.qmth.sop.business.entity.TBQualityProblemApply;
import com.qmth.sop.business.entity.TBService;
import com.qmth.sop.business.mapper.QualityAnalyseMapper;
import com.qmth.sop.business.service.QualityAnalyseService;
import com.qmth.sop.common.enums.InfluenceDegreeEnum;
import com.qmth.sop.common.enums.QualityAnalyseGroupEnum;
import com.qmth.sop.common.enums.QualityProblemReasonEnum;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class QualityAnalyseServiceImpl extends ServiceImpl<QualityAnalyseMapper, TBQualityProblemApply> implements QualityAnalyseService {


    @Override
    public List<TBService> list(Long startTime,Long endTime) {
        return this.baseMapper.list(startTime,endTime);
    }


    @Override
    public Double progress(Long serviceUnitId) {
        return this.baseMapper.progress(serviceUnitId);
    }

    @Override
    public Map<String, Map<String, Object>> influence(Long serviceUnitId, QualityAnalyseGroupEnum group) {
        List<Map<String, Object>> influence = this.baseMapper.influence(serviceUnitId);
        Map<String, Map<String, Object>> result = new HashMap<>();
        if (group.equals(QualityAnalyseGroupEnum.INFLUENCE_REGION)) {
            influence.stream().filter(map->map.get("region_name")!=null).collect(Collectors.groupingBy(map -> map.get("region_name"))).forEach((k, v) -> {
                Map<String, Object> map = new HashMap<>();
                v.stream().filter(map1->map1.get("Influence_degree")!=null).collect(Collectors.groupingBy(map1 -> map1.get("Influence_degree"))).forEach((k1, v1) -> map.put(k1.toString(), v1.size()));
                result.put(k.toString(), map);
            });
        } else if (group.equals(QualityAnalyseGroupEnum.INFLUENCE_SUPPLIER)) {
            influence.stream().filter(map->map.get("Influence_degree")!=null).collect(Collectors.groupingBy(map -> map.get("Influence_degree"))).forEach((k, v) -> {
                Map<String, Object> map = new HashMap<>();
                v.stream().filter(map1->map1.get("supplier")!=null).collect(Collectors.groupingBy(map1 -> map1.get("supplier"))).forEach((k1, v1) -> map.put(k1.toString(), v1.size()));
                result.put(k.toString(), map);
            });
        } else if (group.equals(QualityAnalyseGroupEnum.REASON_REGION)) {
            influence.stream().filter(map->map.get("region_name")!=null).collect(Collectors.groupingBy(map -> map.get("region_name"))).forEach((k, v) -> {
                Map<String, Object> map = new HashMap<>();
                v.stream().filter(map1->map1.get("reason")!=null).collect(Collectors.groupingBy(map1 -> map1.get("reason"))).forEach((k1, v1) -> map.put(k1.toString(), v1.size()));
                result.put(k.toString(), map);
            });
        } else if (group.equals(QualityAnalyseGroupEnum.REASON_SUPPLIER)) {
            influence.stream().filter(map->map.get("reason")!=null).collect(Collectors.groupingBy(map -> map.get("reason"))).forEach((k, v) -> {
                Map<String, Object> map = new HashMap<>();
                v.stream().filter(map1->map1.get("supplier")!=null).collect(Collectors.groupingBy(map1 -> map1.get("supplier"))).forEach((k1, v1) -> map.put(k1.toString(), v1.size()));
                result.put(k.toString(), map);
            });
        }
        return result;
    }


    @Override
    public Map<String, Object> pie(Long serviceUnitId) {
        List<Map<String, Object>> pie = this.baseMapper.pie(serviceUnitId, null);
        Map<String, Object> result = new HashMap<>();
        pie.stream().filter(map1->map1.get("Influence_degree")!=null).collect(Collectors.groupingBy(map -> map.get("Influence_degree"))).forEach((k, v) -> {

            result.put(k.toString(),v.stream().filter(map1->map1.get("count")!=null).mapToLong(map1-> (long) map1.get("count")).sum());

        });
        return result;
    }

    @Override
    public Map<String, Map<String, Object>> radar(Long serviceUnitId, QualityAnalyseGroupEnum group) {


        List<Map<String, Object>> radar = this.baseMapper.radar(serviceUnitId, null);;
        Map<String, Map<String, Object>> result = new HashMap<>();
        radar.stream().filter(map1->map1.get("reason")!=null).collect(Collectors.groupingBy(map -> map.get("reason"))).forEach((k, v) -> {
            Map<String, Object> map = new HashMap<>();
            map.put(k.toString(),v.stream().filter(map1->map1.get("count")!=null).mapToLong(map1-> (long) map1.get("count")).sum());
            result.put("all", map);
        });


        if (group.equals(QualityAnalyseGroupEnum.REASON_REGION)) {

            radar.stream().filter(map1->map1.get("region_name")!=null).collect(Collectors.groupingBy(map -> map.get("region_name"))).forEach((k, v) -> {
                Map<String, Object> z = new HashMap<>();
                v.stream().filter(map1->map1.get("reason")!=null).collect(Collectors.groupingBy(map2 -> map2.get("reason"))).forEach((k1, v1) -> {
                    z.put(k1.toString(),v1.stream().filter(map1->map1.get("count")!=null).mapToLong(map1-> (long) map1.get("count")).sum());
                });
                result.put(k.toString(), z);
            });
        } else if (group.equals(QualityAnalyseGroupEnum.REASON_SUPPLIER)) {
            radar.stream().filter(map1->map1.get("supplier")!=null).collect(Collectors.groupingBy(map -> map.get("supplier"))).forEach((k, v) -> {
                Map<String, Object> z = new HashMap<>();
                v.stream().filter(map1->map1.get("reason")!=null).collect(Collectors.groupingBy(map2 -> map2.get("reason"))).forEach((k1, v1) -> {
                    z.put(k1.toString(),v1.stream().filter(map1->map1.get("count")!=null).mapToLong(map1-> (long) map1.get("count")).sum());
                });
                result.put(k.toString(), z);
            });
        }
        return result;

    }

    @Override
    public IPage<TBQualityProblemApplyResult> detail(Page<Object> objectPage, Long serviceUnitId,InfluenceDegreeEnum degree) {
        return this.baseMapper.detail(objectPage, serviceUnitId, degree);
    }

    @Override
    public IPage<TBQualityProblemApplyResult> influence(Page<Object> tPage, Long serviceUnitId, Long supplierId, Long region_id, InfluenceDegreeEnum influenceDegree, QualityProblemReasonEnum reason) {
        return this.baseMapper.influenceDetail(tPage, serviceUnitId, supplierId,region_id, influenceDegree, reason);
    }
}
