package com.qmth.sop.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.sop.business.bean.result.TBCrmResult;
import com.qmth.sop.business.bean.result.UserArchivesAllocationResult;
import com.qmth.sop.business.bean.result.UserArchivesAllocationSubTotalResult;
import com.qmth.sop.business.entity.SysDevice;
import com.qmth.sop.business.entity.TBService;
import com.qmth.sop.business.mapper.ServiceAnalyseMapper;
import com.qmth.sop.business.service.ServiceAnalyseService;
import com.qmth.sop.business.service.SysDeviceService;
import com.qmth.sop.business.service.TBUserArchivesSupplierService;
import com.qmth.sop.common.contant.SystemConstant;
import com.qmth.sop.common.enums.CrmStatusEnum;
import com.qmth.sop.common.enums.DeviceStatusEnum;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ServiceAnalyseServiceImpl extends ServiceImpl<ServiceAnalyseMapper, TBService> implements ServiceAnalyseService {

    @Resource
    SysDeviceService sysDeviceService;

    @Resource
    TBUserArchivesSupplierService tbUserArchivesSupplierService;

    @Override
    public List<TBService> list(Long startTime,Long endTime) {
        return this.baseMapper.list(startTime,endTime);
    }
    /**
     * 4.服务单元概览
     * ①项目执行进度=服务单元下已完成的派单（研究生派单下SOP已完结；教务处派单下已有完成的SOP，无在执行的SOP。可能存在教务处多个SOP之间空窗期导致数据的误差，这个指标允许少量误差存在）；
     * ②设备出库总量：当前服务单元下已出库且尚未入库的设备总量；
     * ③设备占用率：（设备出库总量÷正常设备总量）×100%；
     * ④在服务人员总数：在执行的SOP现场服务人员数量，包括助理工程师、实施工程师、区域协调人；
     * ⑤现场人员占用率：（在服务人员总数÷认证有效的在服务人员总数）×100%；
     */
    @Override
    public Map<String, Object> overview(Long serviceUnitId) {
        Map<String, Object> map = new HashMap<>();
        map.put("projectProgress", this.baseMapper.projectProgress(serviceUnitId));
        map.put("equipmentOutboundTotal", this.baseMapper.equipmentOutboundTotal(serviceUnitId));
        int totalDevices = sysDeviceService.count(new LambdaQueryWrapper<SysDevice>().eq(SysDevice::getStatus, DeviceStatusEnum.NORMAL));
        map.put("equipmentOccupancyRate",totalDevices==0?0: ((Integer) map.get("equipmentOutboundTotal")) * 100 / totalDevices);
        map.put("servicePersonnelTotal", this.baseMapper.servicePersonnelTotal(serviceUnitId));
        //认证有效的人员总数
        int totalUserArchives = tbUserArchivesSupplierService.countAuthentication();
        map.put("sitePersonnelOccupancyRate",totalUserArchives==0?0: ((Integer) map.get("servicePersonnelTotal")) * 100 / totalUserArchives);
        return map;
    }

    /**
     * ①当前服务单元已划分了区域协调人的项目总数、供应商分布占比；
     */
    @Override
    public Map<String, Object> supplier(Long serviceUnitId) {
        Map<String, Object> map = new HashMap<>();
        map.put("projectCount", this.baseMapper.projectCount(serviceUnitId));
        List<Map<String, Object>> maps = this.baseMapper.supplierDistribution(serviceUnitId);
//        maps = maps.stream().peek(m -> m.put("percent",(Integer) map.get("projectCount")==0?0: ((Long) m.get("count")) * 100 / ((Integer) map.get("projectCount")) + "%")).collect(Collectors.toList());
        map.put("supplierDistribution", maps);
        return map;
    }

    @Override
    public IPage<TBCrmResult> supplierDetail(Page<Object> iPage, Long serviceUnitId, Long supplierId) {
        return this.baseMapper.crmDetail(iPage, serviceUnitId,supplierId);
    }

    @Override
    public  List<Map<String,Object>> supplierRole(Long supplierId) {
        return this.baseMapper.supplierRole(supplierId);
    }

    @Override
    public  List<Map<String,Object>> supplierRegion(Long serviceUnitId, Long supplierId) {
        return this.baseMapper.supplierRegion(serviceUnitId, supplierId);
    }


    @Override
    public List<Map<String, Object>> device(Long serviceUnitId) {
        return this.baseMapper.device(serviceUnitId);
    }

    @Override
    public List<Map<String, Object>> personnel(Long serviceUnitId) {
        return this.baseMapper.personnel(serviceUnitId);
    }


    @Override
    public UserArchivesAllocationSubTotalResult findCrmAllocationSubTotal(Long serviceUnitId, Long regionId, Long supplierId) {

        List<UserArchivesAllocationResult> list = this.baseMapper.findCrmAllocation(serviceUnitId, regionId, supplierId);
        Integer publishedCrmCount = Math.toIntExact(list.stream().filter(e -> CrmStatusEnum.PUBLISH.equals(e.getCrmStatus())).count());
        Integer totalCrmCount = list.size();

        Integer coordinatorQuota = list.stream().mapToInt(UserArchivesAllocationResult::getCoordinatorQuota).sum();
        Integer coordinatorDistributed = list.stream().mapToInt(UserArchivesAllocationResult::getCoordinatorDistributed).sum();

        Integer effectQuota = list.stream().mapToInt(UserArchivesAllocationResult::getEffectQuota).sum();
        Integer effectDistributed = list.stream().mapToInt(UserArchivesAllocationResult::getEffectDistributed).sum();

        Integer assistantQuota = list.stream().mapToInt(UserArchivesAllocationResult::getAssistantQuota).sum();
        Integer assistantDistributed = list.stream().mapToInt(UserArchivesAllocationResult::getAssistantDistributed).sum();

        UserArchivesAllocationSubTotalResult result = new UserArchivesAllocationSubTotalResult();
        result.setPublishedCrmCount(publishedCrmCount);
        result.setTotalCrmCount(totalCrmCount);
        result.setCoordinatorDistributed(coordinatorDistributed);
        result.setCoordinatorQuota(coordinatorQuota);
        result.setEffectDistributed(effectDistributed);
        result.setEffectQuota(effectQuota);
        result.setAssistantDistributed(assistantDistributed);
        result.setAssistantQuota(assistantQuota);
        return result;
    }

    @Override
    public List<UserArchivesAllocationResult> findCrmAllocation(Long serviceUnitId, Long regionId, Long supplierId) {
        return this.baseMapper.findCrmAllocation(serviceUnitId, regionId, supplierId);
    }
}
